/**
 * Copyright (c) Istituto Nazionale di Fisica Nucleare, 2006-2014.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 * This package provides main VOMS Java APIs class and interfaces.
 * For an introductory guide to the use of the API, check the API user manual
 *  <a href="http://italiangrid.github.com/voms-api-java">here</a>.
 * 
 * @author Andrea Ceccanti
 *
 */
package org.italiangrid.voms;