/**
 * Copyright (c) Istituto Nazionale di Fisica Nucleare, 2006-2014.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.italiangrid.voms.store;

/**
 * Represents an object which can be periodically updated according to an update
 * frequency.
 * 
 * @author Andrea Ceccanti
 *
 */
public interface Updateable {

  /**
   * @return the frequency (in milliseconds) currently set for this
   *         {@link Updateable} object.
   */
  public long getUpdateFrequency();

  /**
   * Updates the object.
   */
  public void update();

  /**
   * Cancels the future updates of this {@link Updateable} object.
   */
  public void cancel();

}
